@@ -41,29 +41,135 @@
 
 static Bool   as_X_synchronous_mode = False;
 
+/*************************************************************************/
+/* Scr data members access functions to be used in linking DLLs          */
+int get_screen_width(ScreenInfo *scr)
+{
+	if( scr == NULL ) 
+		scr = ASDefaultScr;
+	return scr->MyDisplayWidth;	
+}	 
+
+int get_screen_height(ScreenInfo *scr)
+{
+	if( scr == NULL ) 
+		scr = ASDefaultScr;
+	return scr->MyDisplayHeight;	
+}	 
+
+int get_screen_current_desk(ScreenInfo *scr)
+{
+	if( scr == NULL ) 
+		scr = ASDefaultScr;
+	return scr->CurrentDesk;	
+}	 
+
+struct MyLook *get_screen_look(ScreenInfo *scr)
+{
+	if( scr == NULL ) 
+		scr = ASDefaultScr;
+	return &(scr->Look);	
+}	 
+
+struct ASImageManager *get_screen_image_manager(ScreenInfo *scr)
+{
+	if( scr == NULL ) 
+		scr = ASDefaultScr;
+	return scr->image_manager;	
+}	 
+
+struct ASFontManager *get_screen_font_manager(ScreenInfo *scr)
+{
+	if( scr == NULL ) 
+		scr = ASDefaultScr;
+	return scr->font_manager;	
+}	 
+
+struct ASVisual *get_screen_visual(ScreenInfo *scr)
+{
+	if( scr == NULL ) 
+		scr = ASDefaultScr;
+	return scr->asv;	
+}	 
+
+
+ScreenInfo *get_current_screen()
+{
+	return ASDefaultScr ;
+}	 
+
+void
+reload_screen_image_manager( ScreenInfo *scr, ASImageManager **old_imageman )
+{
+	char *env_path1 = NULL, *env_path2 = NULL ;
+	if( scr == NULL ) 
+		scr = ASDefaultScr ;
+	if( old_imageman )
+	{
+		*old_imageman = scr->image_manager ;
+	}else if( scr->image_manager )
+      	destroy_image_manager( scr->image_manager, False );
+
+	env_path1 = getenv( "IMAGE_PATH" ) ;
+	env_path2 = getenv( "PATH" );
+	if( env_path1 == NULL ) 
+	{
+		env_path1 = env_path2;
+		env_path2 = NULL ;
+	}
+    scr->image_manager = create_image_manager( NULL, 2.2, Environment->pixmap_path?Environment->pixmap_path:"", env_path1, env_path2, NULL );
+	set_xml_image_manager( scr->image_manager );
+    show_progress("Pixmap Path changed to \"%s:%s:%s\" ...", Environment->pixmap_path?Environment->pixmap_path:"", env_path1?env_path1:"", env_path2?env_path2:"");
+}
+
+/*************************************************************************/
+
 void
 get_Xinerama_rectangles (ScreenInfo * scr)
 {
 #ifdef HAVE_XINERAMA
-	register int  i;
-	XineramaScreenInfo *s;
-
-	if ((s = XineramaQueryScreens (dpy, &(scr->xinerama_screens_num))) != NULL)
+	if (XineramaQueryExtension (dpy, &(scr->XineEventBase), &(scr->XineErrorBase)))
 	{
-		scr->xinerama_screens = safemalloc (sizeof (XRectangle) * scr->xinerama_screens_num);
-		for (i = 0; i < scr->xinerama_screens_num; ++i)
+		register int  i;
+		XineramaScreenInfo *s;
+
+		if ((s = XineramaQueryScreens (dpy, &(scr->xinerama_screens_num))) != NULL)
 		{
-			scr->xinerama_screens[i].x = s[i].x_org;
-			scr->xinerama_screens[i].y = s[i].y_org;
-			scr->xinerama_screens[i].width = s[i].width;
-			scr->xinerama_screens[i].height = s[i].height;
+			scr->xinerama_screens = safemalloc (sizeof (XRectangle) * scr->xinerama_screens_num);
+	    	asxml_var_insert("xroot.xinerama_screens_num", scr->xinerama_screens_num);
+			static char buf[256] ; 
+			for (i = 0; i < scr->xinerama_screens_num; ++i)
+			{
+				char *append_point = &buf[0]; 
+				sprintf( append_point, "xroot.xinerama_screens[%d].", i );
+				append_point += 23 ; 
+				while( *append_point ) ++append_point ; 
+
+				append_point[0] = 'x' ;append_point[1] = '\0' ;
+		    	asxml_var_insert(&buf[0], s[i].x_org);
+				scr->xinerama_screens[i].x = s[i].x_org;
+
+				append_point[0] = 'y' ;
+		    	asxml_var_insert(&buf[0], s[i].y_org);
+				scr->xinerama_screens[i].y = s[i].y_org;
+
+				strcpy( append_point, "width" );
+		    	asxml_var_insert(&buf[0], s[i].width);
+				scr->xinerama_screens[i].width = s[i].width;
+
+				strcpy( append_point, "height" );
+		    	asxml_var_insert(&buf[0], s[i].height);
+				scr->xinerama_screens[i].height = s[i].height;
+			}
+			XFree (s);
 		}
-		XFree (s);
-	}
-#else
-	scr->xinerama_screens = NULL;
-	scr->xinerama_screens_num = 0;
+	}else
 #endif
+	{
+		scr->xinerama_screens = NULL;
+		scr->xinerama_screens_num = 0;
+    	asxml_var_insert("xroot.xinerama_screens_num", 0);
+	}
 }
 
 Bool
@@ -144,9 +250,9 @@ ASErrorHandler (Display * dpy, XErrorEvent * event)
 	fprintf (stderr, "%s has encountered a problem interacting with X Windows :\n", MyName);
 	if (event && dpy)
 	{
-        if( event->error_code == BadWindow && Scr.Windows != NULL )
-            if( Scr.on_dead_window )
-                if( Scr.on_dead_window( event->resourceid ) )
+        if( event->error_code == BadWindow && ASDefaultScr->Windows != NULL )
+            if( ASDefaultScr->on_dead_window )
+                if( ASDefaultScr->on_dead_window( event->resourceid ) )
                     return 0;
 
         err_text = safemalloc (128);
@@ -164,15 +270,36 @@ ASErrorHandler (Display * dpy, XErrorEvent * event)
 }
 
 
-int
-ConnectX (ScreenInfo * scr, unsigned long event_mask)
+void 
+init_ScreenInfo(ScreenInfo *scr) 
 {
-    /* Initialize X connection */
-    if (!(dpy = XOpenDisplay (MyArgs.display_name)))
+	if( scr )
 	{
-        show_error("Can't open display \"%s\". Exiting!", XDisplayName (MyArgs.display_name));
-		exit (1);
+		memset (scr, 0x00, sizeof (ScreenInfo));
+		scr->Look.magic = MAGIC_MYLOOK ;
 	}
+}
+
+int
+ConnectXDisplay (Display *display, ScreenInfo * scr, Bool as_manager)
+{
+    int width_mm, height_mm ;
+	int display_dpcmx = 0, display_dpcmy = 0;
+	int button_w, button_h, mini_w, mini_h ;
+	
+	if( display == NULL ) 
+		return -1;
+
+	dpy = display ;
+	
+	if( scr == NULL ) 
+	{
+		if( ASDefaultScr == NULL ) 
+			ASDefaultScr = safecalloc( 1, sizeof(ScreenInfo));
+		scr = ASDefaultScr ;
+	}
+
+	init_ScreenInfo(scr);	
 
 	x_fd = XConnectionNumber (dpy);
 
@@ -192,8 +319,6 @@ ConnectX (ScreenInfo * scr, unsigned long event_mask)
     intern_hint_atoms ();
     intern_wmprop_atoms ();
 
-	memset (scr, 0x00, sizeof (ScreenInfo));
-
 	scr->screen = DefaultScreen (dpy);
 	scr->Root = RootWindow (dpy, scr->screen);
 	if (scr->Root == None)
@@ -209,12 +334,34 @@ ConnectX (ScreenInfo * scr, unsigned long event_mask)
 	asxml_var_insert("xroot.width", scr->MyDisplayWidth);
     asxml_var_insert("xroot.height", scr->MyDisplayHeight);
 
+	width_mm = DisplayWidthMM(dpy, scr->screen);
+	height_mm = DisplayHeightMM(dpy, scr->screen);
+	asxml_var_insert("xroot.widthmm", width_mm);
+    asxml_var_insert("xroot.heightmm", height_mm);
+	
+	display_dpcmx = (scr->MyDisplayWidth * 10 )/ width_mm;
+	display_dpcmy = (scr->MyDisplayHeight * 10 )/ height_mm;
+
+	button_w = (display_dpcmx < 50)?48:64;
+	button_h = (display_dpcmy < 50)?48:64;
+	mini_w = (display_dpcmx <= 44)?max(display_dpcmx/2,12):display_dpcmx-20;
+	mini_h = (display_dpcmy <= 44)?max(display_dpcmy/2,12):display_dpcmy-20;
+	asxml_var_insert(ASXMLVAR_IconButtonWidth, button_w);
+	asxml_var_insert(ASXMLVAR_IconButtonHeight, button_h);
+	asxml_var_insert(ASXMLVAR_IconWidth, (button_w*3)/4);
+	asxml_var_insert(ASXMLVAR_IconHeight, (button_h*3)/4);
+	asxml_var_insert(ASXMLVAR_MinipixmapWidth, mini_w);
+	asxml_var_insert(ASXMLVAR_MinipixmapHeight, mini_h);
+	asxml_var_insert(ASXMLVAR_TitleFontSize, (mini_h*7)/12);
+	asxml_var_insert(ASXMLVAR_MenuFontSize, (mini_h*7)/12+1);
+	asxml_var_insert("font.size", ((mini_h*7)/12));
+
 	scr->CurrentDesk = -1;
 
     scr->RootClipArea.width = scr->MyDisplayWidth;
     scr->RootClipArea.height = scr->MyDisplayHeight;
 
-    if( (scr->wmprops = setup_wmprops( scr, (event_mask&SubstructureRedirectMask), 0xFFFFFFFF, NULL )) == NULL )
+    if( (scr->wmprops = setup_wmprops( scr, as_manager, 0xFFFFFFFF, NULL )) == NULL )
         return -1;
 
     scr->asv = create_asvisual (dpy, scr->screen, DefaultDepth(dpy,scr->screen), NULL);
@@ -231,10 +378,8 @@ ConnectX (ScreenInfo * scr, unsigned long event_mask)
 
     setup_modifiers ();
 
-#ifdef HAVE_XINERAMA
-	if (XineramaQueryExtension (dpy, &(scr->XineEventBase), &(scr->XineErrorBase)))
-		get_Xinerama_rectangles (scr);
-#endif
+	get_Xinerama_rectangles (scr);
+
 #ifdef SHAPE
 	XShapeQueryExtension (dpy, &(scr->ShapeEventBase), &(scr->ShapeErrorBase));
 #endif /* SHAPE */
@@ -242,13 +387,26 @@ ConnectX (ScreenInfo * scr, unsigned long event_mask)
 	scr->ShmCompletionEventType = XShmGetEventBase(dpy) + ShmCompletion;
 #endif /* SHAPE */
 
-    XSelectInput (dpy, scr->Root, event_mask);
-
 	init_screen_gcs(scr);
 
 	return x_fd;
 }
 
+int
+ConnectX (ScreenInfo * scr, unsigned long event_mask)
+{
+    if (!(dpy = XOpenDisplay (MyArgs.display_name)))
+	{
+        show_error("Can't open display \"%s\". Exiting!", XDisplayName (MyArgs.display_name));
+		exit (1);
+	}
+
+	ConnectXDisplay (dpy, scr, (event_mask&SubstructureRedirectMask));
+    XSelectInput (dpy, scr->Root, event_mask);
+	return x_fd;
+}
+
+
 /***********************************************************************
  *  Procedure:
  *  SetupEnvironment - setup environment variables DISPLAY and HOSTDISPLAY
@@ -264,7 +422,7 @@ make_screen_envvars( ScreenInfo *scr )
  * as some OS's don't copy the environment variables properly
  */
 	if( scr == NULL )
-		return;
+		scr = ASDefaultScr;
 
 	if( scr->display_string )
         free( scr->display_string );
@@ -302,18 +460,22 @@ make_screen_envvars( ScreenInfo *scr )
 void
 init_screen_gcs(ScreenInfo *scr)
 {
-    if( scr )
-	{
-		XGCValues     gcv;
-    	unsigned long gcm = GCGraphicsExposures;
-		gcv.graphics_exposures = False;
-        scr->DrawGC = create_visual_gc( scr->asv, scr->Root, gcm, &gcv );
-    }
+	XGCValues     gcv;
+   	unsigned long gcm = GCGraphicsExposures;
+	
+	if( scr == NULL )
+		scr = ASDefaultScr;
+	
+	gcv.graphics_exposures = False;
+    scr->DrawGC = create_visual_gc( scr->asv, scr->Root, gcm, &gcv );
 }
 
 void
 destroy_screen_gcs(ScreenInfo *scr)
 {
+	if( scr == NULL )
+		scr = ASDefaultScr;
+	
 	if( scr->DrawGC )
 	{	
 		XFreeGC(dpy, scr->DrawGC);
@@ -388,24 +550,27 @@ init_screen_panframes(ScreenInfo *scr)
     XSetWindowAttributes attributes;           /* attributes for create */
     register int i ;
 
+	if( scr == NULL ) 
+		scr =  ASDefaultScr ;
+
     frame_rects[0].x = frame_rects[0].y = 0;
-    frame_rects[0].width = Scr.MyDisplayWidth;
+    frame_rects[0].width = scr->MyDisplayWidth;
     frame_rects[0].height = SCROLL_REGION;
 
-    frame_rects[1].x = Scr.MyDisplayWidth - SCROLL_REGION;
+    frame_rects[1].x = scr->MyDisplayWidth - SCROLL_REGION;
     frame_rects[1].y = SCROLL_REGION;
     frame_rects[1].width = SCROLL_REGION;
-    frame_rects[1].height = Scr.MyDisplayHeight - 2 * SCROLL_REGION;
+    frame_rects[1].height = scr->MyDisplayHeight - 2 * SCROLL_REGION;
 
     frame_rects[2].x = 0;
-    frame_rects[2].y = Scr.MyDisplayHeight - SCROLL_REGION;
-    frame_rects[2].width = Scr.MyDisplayWidth;
+    frame_rects[2].y = scr->MyDisplayHeight - SCROLL_REGION;
+    frame_rects[2].width = scr->MyDisplayWidth;
     frame_rects[2].height = SCROLL_REGION;
 
     frame_rects[3].x = 0;
     frame_rects[3].y = SCROLL_REGION;
     frame_rects[3].width = SCROLL_REGION;
-    frame_rects[3].height = Scr.MyDisplayHeight - 2 * SCROLL_REGION;
+    frame_rects[3].height = scr->MyDisplayHeight - 2 * SCROLL_REGION;
 
     frame_rects[2].width = frame_rects[0].width = scr->MyDisplayWidth;
     frame_rects[1].x = scr->MyDisplayWidth - SCROLL_REGION;
@@ -442,7 +607,8 @@ check_screen_panframes(ScreenInfo *scr)
     register int  i;
 
 	if( scr == NULL )
-		scr = &Scr ;
+		scr = ASDefaultScr;
+ ;
 
     wrapX = get_flags(scr->Feel.flags, EdgeWrapX);
     wrapY = get_flags(scr->Feel.flags, EdgeWrapY);
@@ -503,7 +669,7 @@ raise_scren_panframes (ScreenInfo *scr)
 #ifndef NO_VIRTUAL
     register int i ;
 	if( scr == NULL )
-		scr = &Scr ;
+		scr = ASDefaultScr ;
     for( i = 0 ; i < PAN_FRAME_SIDES ; i++ )
         if( scr->PanFrame[i].isMapped )
         {
@@ -517,6 +683,8 @@ Window
 get_lowest_panframe(ScreenInfo *scr)
 {
     register int i;
+	if( scr == NULL )
+		scr = ASDefaultScr ;
     for( i = 0 ; i < PAN_FRAME_SIDES ; i++ )
         if( scr->PanFrame[i].isMapped )
             return scr->PanFrame[i].win;
@@ -549,7 +717,7 @@ void
 check_desksize_sanity( ScreenInfo *scr )
 {
 	if( scr == NULL )
-		scr = &Scr ;
+		scr = ASDefaultScr ;
 
     if( scr->VxMax <= 0 )
         scr->VxMax = 0 ;
@@ -559,6 +727,6 @@ check_desksize_sanity( ScreenInfo *scr )
     if( scr->VyMax <= 0 )
         scr->VyMax = 0 ;
     else if( scr->VyMax < 32000/scr->MyDisplayHeight )
-        scr->VyMax = (Scr.VyMax * scr->MyDisplayHeight) - scr->MyDisplayHeight ;
+        scr->VyMax = (scr->VyMax * scr->MyDisplayHeight) - scr->MyDisplayHeight ;
 }
 
