@@ -121,15 +121,16 @@ LOCAL_DEBUG_OUT( "module name \"%s\"", module->name );
     	while (module->output_queue != NULL)
         	DeleteQueueBuff (module);
     	if (module->name != NULL)
-        	free (module->name);
-    	if (module->ibuf.text != NULL)
-        	free (module->ibuf.text);
+        destroy_string( &(module->name));
+        destroy_string( &(module->cmd_line));
+    	destroy_string( &(module->ibuf.text));
 
     	if (module->ibuf.func != NULL)
 		{
         	free_func_data (module->ibuf.func);
         	free (module->ibuf.func);
 		}
+		
 
     	memset( module, 0x00, sizeof(module_t) );
 	}else
@@ -554,7 +555,7 @@ int
 AcceptModuleConnection (int socket_fd)
 {
 	int           fd;
-	int           len = sizeof (struct sockaddr_un);
+	unsigned int  len = sizeof (struct sockaddr_un);
 	struct sockaddr_un name;
 
 	fd = accept (socket_fd, (struct sockaddr *)&name, &len);
@@ -839,10 +840,10 @@ SendConfig (int module, send_data_type event_type, ASWindow * t)
     }
 
 	asw_id.asw = t ;
-    SendPacket (module, event_type, 25,
+    SendPacket (module, event_type, 26,
                 t->w,                 t->frame,              asw_id.id,
                 frame_x,              frame_y,               frame_width,         frame_height,
-                ASWIN_DESK(t),        t->status->flags,      t->hints->flags,
+                ASWIN_DESK(t),        t->status->flags,      t->hints->flags,     t->hints->client_icon_flags,
                 t->hints->base_width, t->hints->base_height, t->hints->width_inc, t->hints->height_inc,
                 t->hints->min_width,  t->hints->min_height,  t->hints->max_width,  t->hints->max_height,
                 t->hints->gravity,    icon_title_w,          icon_pixmap_w,
@@ -1144,7 +1145,20 @@ make_module_menu( FunctionCode func, const char *title, int sort_order )
 				minipixmap = de->fulliconname ;
 
 			if( (mdi = add_menu_fdata_item( md, menuitems[i], minipixmap, NULL)) != NULL ) 
+			{
 				set_flags( mdi->flags, MD_ScaleMinipixmapDown );
+				if( de )
+				{
+					char *comment = NULL; 
+					if( dup_desktop_entry_Comment( de, &comment ) )
+						set_flags( mdi->flags, MD_CommentIsUTF8 );
+					if( comment )
+					{
+						mdi->comment = interpret_ascii_string( comment );
+						free( comment );
+					}
+				}
+			}
 			safefree( menuitems[i] ); /* scrubba-dub-dub */
 		}
 		safefree(menuitems);
